package com.jh.boot.attachment.api;

import java.io.InputStream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.jh.boot.attachment.Attachment;
import com.jh.boot.attachment.AttachmentService;
import com.jh.boot.web.error.NotFoundException;

@Component
public class AttachmentContentProvider {
	
	private AttachmentService service;

	
    public ResponseEntity<InputStreamResource> fetch(Long attachmentId) {
    	Attachment a = service.fetchAttachment(attachmentId);
    	if (a == null) {
    		throw new NotFoundException();
    	}
    	
    	InputStream is = service.fetchAttachmentBody(a);
    	if (is == null) {
    		throw new NotFoundException();
    	}
    	
    	HttpHeaders headers = new HttpHeaders();
    	headers.setContentType(StringUtils.hasText(a.getMimeType()) ? MediaType.parseMediaType(a.getMimeType()) : MediaType.APPLICATION_OCTET_STREAM);
    	headers.set("Content-Disposition", "attachment; filename=\"" + a.getFilename() + "\"");

    	return new ResponseEntity<>(new InputStreamResource(is), headers, HttpStatus.OK);
    }


    @Autowired(required = false)
	public void setService(AttachmentService service) {
		this.service = service;
	}
	
}
