package com.jh.boot.jpa;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.Consumer;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.hibernate.query.NativeQuery;

import com.jh.boot.web.list.Page;
import com.jh.boot.web.list.PagingInfo;


/**
 * TODO comment.
 *
 * @author jh
 */
public abstract class AbstractHibernateRepository {

    /** The session factory. */
    @PersistenceContext
    protected EntityManager entityManager;



    /**
     * Criteria.
     *
     * @param clazz the clazz
     * @return the criteria
     */
    protected <T> CriteriaQuery<T> criteria(Class<T> clazz) {
    	return entityManager.getCriteriaBuilder().createQuery(clazz);
    }
    
    
    @SuppressWarnings("unchecked")
	protected <T> NativeQuery<T> nativeQuery(String query) {
    	return (NativeQuery<T>) entityManager.createNativeQuery(query);
    }
    
    @SuppressWarnings("unchecked")
	protected <T> T singleResult(Query q) {
    	try {
    		return (T) q.getSingleResult();
    	} catch (NoResultException e) {
    		return null;
    	}
    }

    
	public <ID, T> T findById(Class<T> clazz, ID id) {
		return entityManager.find(clazz, id);
	}
	
	
	/**
	 * TODO.
	 *
	 * @param <T> the generic type
	 * @param entity the entity
	 */
    public <T extends AbstractIdEntity> void save(T entity) {
        if (entity.getId() == null) {
            entityManager.persist(entity);
        } else {
            entityManager.merge(entity);
        }
    }

    /**
     * Delete.
     *
     * @param <T> the generic type
     * @param entity the entity
     */
    public <T extends AbstractIdEntity> void delete(T entity) {
        if (entity.getId() != null) {
        	entityManager.remove(entity);
        }
    }


	protected <T extends AbstractIdEntity> void synchronizeCollections(Collection<T> currentCollection, Collection<T> newCollection, BiConsumer<T, T> modifyFn, Consumer<T> modifyRelationship, boolean deleteOrphans) {
		Objects.requireNonNull(currentCollection);
		Objects.requireNonNull(newCollection);
		Objects.requireNonNull(modifyFn);
		
		List<T> toBeAdded = new ArrayList<>(); 
		Map<Long, T> map = new HashMap<>();
		if (newCollection != null) {
			for (T nc : newCollection) {
				if (nc.getId() == null) {
					toBeAdded.add(nc);
				} else {
					map.put(nc.getId(), nc);
				}
			}
		}
		
		Map<Long, T> newMap = map;
		
		Iterator<T> it = currentCollection.iterator();
		while (it.hasNext()) {
			T t = it.next();
			T nc = newMap.get(t.getId());
			if (nc == null) {
				it.remove();
				if (deleteOrphans) {
					delete(t);
				}
				continue;
			}

			modifyFn.accept(t, nc);
		}
		
		for (T tba : toBeAdded) {
			save(tba);
		}
		
	}


	@SuppressWarnings("unchecked")
	protected <T> Page<T> pagedCriteria(CriteriaBuilder cb, CriteriaQuery<T> cq, Root<T> root, PagingInfo pagingInfo) {
		Query query = entityManager.createQuery(cq);
		query.setFirstResult(pagingInfo.getPage() * pagingInfo.getPageSize());
		query.setMaxResults(pagingInfo.getPageSize());
		List<T> resultList = query.getResultList();

		CriteriaQuery<Long> count = cb.createQuery(Long.class);
		Root<T> countRoot = count.from(root.getModel());
		count.select(cb.count(countRoot));
		if (cq.getRestriction() != null) {
			count.where(cq.getRestriction());
		}
		Long rowsCount = entityManager.createQuery(count).getSingleResult();

		return new Page<T>(pagingInfo.getPage(), pagingInfo.getPageSize(), rowsCount, resultList, pagingInfo.getField(), pagingInfo.getTrend());
	}
    
}
