package com.jh.boot.security;

import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

import com.jh.boot.security.model.AppUser;

public class SecurityHelper {
	
	private SecurityHelper() {
	}

	
	public static AppUser getUser() {
		return (AppUser) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
	}
	
	
	public static AppUser getUser(Authentication auth) {
		if (auth == null) {
			return null;
		}
		
		return (AppUser) auth.getPrincipal();
	}
	
	
	public static <T extends AppUser> T getUser(Authentication auth, Class<T> clazz) {
		if (AppUser.class.isAssignableFrom(clazz)) {
			return (T) auth.getPrincipal();
		}
		
		return null;
	}
	
	
	public static boolean hasAnyRole(Authentication auth, String...roles) {
		AppUser user = auth instanceof AppUserAuthentication ? ((AppUserAuthentication) auth).getUser() : null;
		return hasAnyRole(user, roles);
	}


	public static boolean hasAnyRole(AppUser user, String... roles) {
		if (user == null || user.getRolesMap() == null || roles == null || roles.length == 0) {
			return false;
		}
		
		for (String role : roles) { 
			if (user.getRolesMap().containsKey(role)) {
				return true;
			}
		}
		
		return false;
	}
	
	
}
